% MOST OF THE CODE RELATED TO POLAR CODES IS ADAPTED FROM 
% https://ecse.monash.edu/staff/eviterbo/polarcodes.html,
% "Polar Codes for Matlab" (Vangala et al)

clear all

% Simulation parameters 
nV = 2.^([8,9,10,11,12,13,14,15,16,17,18,19,20,21,22]); %blocklengths
beta_s = 0.3; % beta_s (\delta_n = 2^{-n^{\beta}})
beta_r = 0.36; % beta_r 
rho_R = 80; % normalized target rate


% CHANNEL : BE-BC (2 leg. rx and 2 eves)
Nlegch = 2;
Nevech = 2;
a2 = 0.01;
a1 = 0.04;
b2 = 0.2;
b1 = 0.35; 

% ----------------------------------------------
% Add path:
addpath('functions')


% Optimum input distribution - Individual Channels - Initial parameters for PCC:
% Channels{1} : X-Y2  ||? Channels{2} : X-Y1  ||? Channels{3} : X-Z2  ||? Channels{2} : X-Z1 
pxst = 0.5; % p_x(x^{\star}) = 0.5.
p_ch = zeros(1,Nlegch+Nevech);
channelnames = cell(1,Nlegch+Nevech);
for ind = 1:Nlegch
    aux = ['a',num2str(Nlegch-ind+1)];
    auxstr = num2str(eval(aux));
    channelnames{ind} = [aux,'_',auxstr(3:min(end,7))];
    p_ch(ind) = eval(aux);
end
for ind = 1:+Nevech
    aux = ['b',num2str(Nevech-ind+1)];
    auxstr = num2str(eval(aux));
    channelnames{Nlegch+ind} = [aux,'_',auxstr(3:min(end,6))];
    p_ch(Nlegch+ind) = eval(aux);
end
% Capacity : (R1t,R2t)
[R1t,R2t,Channels,ChannelsCell] = fun1ev(pxst,p_ch);

NameCh = [];
for ind = 1:length(channelnames)-1
    NameCh = [NameCh,channelnames{ind},'_']; %#ok<AGROW>
end
NameCh = [NameCh,channelnames{4}];

% Target rates in which polar coding scheme operates:
R1targ = (rho_R/100) * R1t;
R2targ = (rho_R/100) * R2t;

% Filename (where results are stored) 
filename = ['Simulated/M1_PercentRate_',num2str(rho_R),'_betaS(percent)_',num2str(beta_s*100),'_betaR(percent)_',num2str(beta_r*100),'_from_',num2str(log2(nV(1))),'_to_',num2str(log2(nV(end)))];

% Information leakages (upper bound):
I_I1_Z1 = zeros(1,length(nV));
I_I2_Z1 = zeros(1,length(nV));
I_F_Z1 = zeros(1,length(nV));
I_I1I2_Z1 = zeros(1,length(nV));
I_I1I2F_Z1 = zeros(1,length(nV));
I_I2_Z2 = zeros(1,length(nV));
I_F_Z2 = zeros(1,length(nV));
I_I2F_Z2 = zeros(1,length(nV));

% Average Bit Error Probability (upper bound based on Bhat. parameters)
Pe_I1I2_Y1 = zeros(1,length(nV));
Pe_I1I2_Y2 = zeros(1,length(nV));

% Rate of the additional secret sequence
SSR = zeros(1,length(nV));

for ind = 1:length(nV),
    tic;
    
    % blocklength
    n = nV(ind);
    
    % delta_ns (function of betas)
    delta_n_s = (-n^(beta_s))*log(2);
    delta_n_r = (-n^(beta_r))*log(2);
    
    % Polar code construction (Computation of Bhat. parameters)
    % Exact values for BE-BC
    [ChannelOutput] = polar_code_construction_log(ChannelsCell,Channels,n); 
        
    % Define sets according to the computed Bhat. parameters:
    % (partition of the universal set [n])
    % ISS -> indices of the additional secret sequence
    [I1,I2,F,C,ISS] = define_sets_new_log(ChannelOutput,ceil(R1targ*n),ceil(R2targ*n),delta_n_s,delta_n_r);
    
    % Information leakage upper-bounds:
    % From Bhatt. parameters
    I_I1_Z1(ind) = length(I1) - sum(exp(2*ChannelOutput{4}.Polarization.Z(I1)));
    I_I2_Z1(ind) = length(I2) - sum(exp(2*ChannelOutput{4}.Polarization.Z(I2)));
    I_F_Z1(ind) = length(F) - sum(exp(2*ChannelOutput{4}.Polarization.Z(F)));
    I_I1I2_Z1(ind) = I_I1_Z1(ind) + I_I2_Z1(ind);
    I_I1I2F_Z1(ind) = I_I1_Z1(ind) + I_I2_Z1(ind) + I_F_Z1(ind);
    I_I2_Z2(ind) = length(I2) - sum(exp(2*ChannelOutput{3}.Polarization.Z(I2)));
    I_F_Z2(ind) = length(F) - sum(exp(2*ChannelOutput{3}.Polarization.Z(F)));
    I_I2F_Z2(ind) = I_I2_Z2(ind) + I_F_Z2(ind);
    
    % Error probability upper-bounds:
    H_XY1_compl = union(C,union(I1,I2));
    L_XY1 = setdiff(H_XY1_compl,ISS);
    Pe_I1I2_Y1(ind) = sum(exp(ChannelOutput{2}.Polarization.Z(L_XY1)))/length(L_XY1);
    Pe_I1I2_Y2(ind) = sum(exp(ChannelOutput{1}.Polarization.Z(L_XY1)))/length(L_XY1);
    
    % Additional secret sequence rate:
    SSR(ind) = length(ISS)/n;
    
    t = toc;
    fprintf('Done for n=2^N, where N equal to %d - %.2f seconds .\n',log2(n),t);
    
end
save(filename,'I_I1I2F_Z1','I_I2F_Z2','Pe_I1I2_Y1','Pe_I1I2_Y2','SSR');
    